@@ -8,7 +8,7 @@ from scope_plot import schema
 
 
 class InputFileNotFoundError(Exception):
-    """raise when a spec file does not define 'backend'"""
+    """raise when an input_file cannot be found in include directories"""
     def __init__(self, name, search_dirs):
         self.name = name
         self.search_dirs = search_dirs
@@ -17,6 +17,20 @@ class InputFileNotFoundError(Exception):
         return "input_file {} not found in any of {}".format(self.name, self.search_dirs)
 
 
+class InputFileNotDefinedError(Exception):
+    """raise when a spec file does not define input_file"""
+    def __init__(self, series):
+        self.series = series
+
+
+class XfieldNotFoundError(Exception):
+    """raise when xfield is not defined"""
+
+
+class YfieldNotFoundError(Exception):
+    """raise when yfield is not defined"""
+
+
 def find(name, search_dirs):
     if not os.path.isfile(name):
         found = False
@@ -30,6 +44,96 @@ def find(name, search_dirs):
             return None
 
 
+class input_file_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._input_file = spec.get("input_file", None)
+
+    def input_file(self):
+        if self._input_file:
+            return self._input_file
+        elif isinstance(self.parent, input_file_mixin):
+            return self.parent.input_file()
+        else:
+            raise InputFileNotDefinedError(self)
+
+
+class regex_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._regex = spec.get("regex", None)
+
+    def regex(self):
+        if self._regex:
+            return self._regex
+        elif isinstance(self.parent, regex_mixin):
+            return self.parent.regex()
+        else:
+            return ".*"
+
+
+class xfield_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._xfield = spec.get("xfield", None)
+
+    def xfield(self):
+        if self._xfield:
+            return self._xfield
+        elif isinstance(self.parent, xfield_mixin):
+            f = self.parent.xfield()
+            if not f:
+                raise XfieldNotFoundError
+            return f
+        return None
+
+
+class yfield_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._yfield = spec.get("yfield", None)
+
+    def yfield(self):
+        if self._yfield:
+            return self._yfield
+        elif isinstance(self.parent, yfield_mixin):
+            f = self.parent.yfield()
+            if not f:
+                raise YfieldNotFoundError
+            return f
+        return None
+
+
+class xscale_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._xscale = spec.get("xscale", None)
+
+    def xscale(self):
+        if self._xscale:
+            return eval(str(self._xscale))
+        elif isinstance(self.parent, xscale_mixin):
+            f = self.parent.xscale()
+            if f:
+                return f
+        return 1.0
+
+
+class yscale_mixin(object):
+    def __init__(self, parent, spec):
+        self.parent = parent
+        self._yscale = spec.get("yscale", None)
+
+    def yscale(self):
+        if self._yscale:
+            return eval(str(self._yscale))
+        elif isinstance(self.parent, yscale_mixin):
+            f = self.parent.yscale()
+            if f:
+                return f
+        return 1.0
+
+
 class SpecificationBase(object):
     """ emulate a dictionary to provide compatibility with most of old implementation"""
     def __init__(self, parent, spec):
@@ -52,57 +156,80 @@ class SpecificationBase(object):
         return self.spec.get(key, default)
 
 
-class SeriesSpecification(SpecificationBase):
+class SeriesSpecification(
+    input_file_mixin,
+    SpecificationBase,
+    regex_mixin,
+    xfield_mixin,
+    xscale_mixin,
+    yfield_mixin,
+    yscale_mixin,
+):
     def __init__(self, parent, spec):
-        super(SeriesSpecification, self).__init__(parent, spec)
-        self._input_file = spec.get("input_file", None)
+        SpecificationBase.__init__(self, parent, spec)
+        input_file_mixin.__init__(self, parent, spec)
+        regex_mixin.__init__(self, parent, spec)
+        xfield_mixin.__init__(self, parent, spec)
+        yfield_mixin.__init__(self, parent, spec)
+        xscale_mixin.__init__(self, parent, spec)
+        yscale_mixin.__init__(self, parent, spec)
+        self._color = spec.get("color", None)
+        self._label = spec.get("label", None)
 
     def label_seperator(self):
-        """the seperator that should be used to build the label, or None if the label a string"""
-        label_spec = self.spec["label"]
-        if isinstance(label_spec, dict):
-            return label_spec.get("seperator", "x")
-        return None
+        """the seperator that should be used to build the label, or x if not defined"""
+        if isinstance(self._label, dict):
+            return self._label.get("seperator", "x")
+        return "x"
 
     def label_fields(self):
-        """the fields that should be used to build the label, or None if the label a string"""
-        label_spec = self.spec["label"]
-        if isinstance(label_spec, dict):
-            return label_spec["fields"]
-        return None
+        """the benchmark fields that should be used to build the label, or [] if not defined"""
+        if isinstance(self._label, dict):
+            return self._label.get("fields", [])
+        return []
 
-    def label(self):
-        """return the label, if it is a string, or None"""
-        label_spec = self.spec["label"]
-        if isinstance(label_spec, str):
-            return label_spec
-        return None
+    def label_or(self, default=None):
+        """return the series label, if it is a string, the default"""
+        if isinstance(self._label, str):
+            return self._label
+        return default
 
     def find_input_file(self, search_dirs):
         if self._input_file:
-            utils.debug("searching for input_file={} defined for series".format(self.input_file))
+            utils.debug("searching for input_file={} defined for series".format(self._input_file))
             e = InputFileNotFoundError(self._input_file, search_dirs)
             self._input_file = find(self._input_file, search_dirs)
             if not self._input_file:
                 raise e
 
-    def input_file(self):
-        if self._input_file:
-            return self._input_file
-        f = self.parent.input_file()
-        assert f
-        return f
+    def color_or(self, default=None):
+        """return series color, or default value if color not defined"""
+        if self._color:
+            return self._color
+        return default
 
 
-class PlotSpecification(SpecificationBase):
+class PlotSpecification(
+    SpecificationBase,
+    input_file_mixin,
+    regex_mixin,
+    xfield_mixin,
+    xscale_mixin,
+    yfield_mixin,
+    yscale_mixin,
+):
     def __init__(self, parent, spec):
-        super(PlotSpecification, self).__init__(parent, spec)
+        SpecificationBase.__init__(self, parent, spec)
+        input_file_mixin.__init__(self, parent, spec)
+        regex_mixin.__init__(self, parent, spec)
+        xfield_mixin.__init__(self, parent, spec)
+        yfield_mixin.__init__(self, parent, spec)
+        xscale_mixin.__init__(self, parent, spec)
+        yscale_mixin.__init__(self, parent, spec)
         self.series = [
             SeriesSpecification(self, s) for s in spec["series"]
         ]
-        self._input_file = spec.get("input_file", None)
         self.type_str = spec.get("type", None)
-        self.spec = spec
 
     def find_input_files(self, search_dirs):
         if self._input_file:
@@ -110,16 +237,9 @@ class PlotSpecification(SpecificationBase):
             self._input_file = find(self._input_file, search_dirs)
             if not self._input_file:
                 raise e
-
         for series in self.series:
             series.find_input_file(search_dirs)
 
-    def input_file(self):
-        """ input_file for plot """
-        if self._input_file:
-            return self._input_file
-        return self.parent.input_file()
-
     def ty(self):
         """ plot type """
         if self.type_str:
@@ -129,9 +249,23 @@ class PlotSpecification(SpecificationBase):
         return type_str
 
 
-class Specification(SpecificationBase):
+class Specification(
+    SpecificationBase,
+    input_file_mixin,
+    regex_mixin,
+    xfield_mixin,
+    xscale_mixin,
+    yfield_mixin,
+    yscale_mixin
+):
     def __init__(self, spec):
-        super(Specification, self).__init__(parent=None, spec=spec)
+        SpecificationBase.__init__(self, parent=None, spec=spec)
+        input_file_mixin.__init__(self, None, spec)
+        regex_mixin.__init__(self, None, spec)
+        xfield_mixin.__init__(self, None, spec)
+        yfield_mixin.__init__(self, None, spec)
+        xscale_mixin.__init__(self, None, spec)
+        yscale_mixin.__init__(self, None, spec)
         if "subplots" in spec:
             self.subplots = [
                 PlotSpecification(self, s) for s in spec["subplots"]
@@ -140,7 +274,6 @@ class Specification(SpecificationBase):
             utils.debug("subplot not in spec")
             self.subplots = [PlotSpecification(self, spec)]
             self.subplots[0]["pos"] = (1, 1)
-        self._input_file = spec.get("input_file", None)
         self.size = spec.get("size", None)
         self.type_str = spec.get("type", None)
 
@@ -150,17 +283,13 @@ class Specification(SpecificationBase):
             for series in plot.series:
                 yield series.input_file()
 
-    def input_file(self):
-        return self._input_file
-
     def find_input_files(self, search_dirs):
         if self._input_file:
-            utils.debug("searching for input_file={} defined at top level of spec".format(self.input_file))
+            utils.debug("searching for input_file={} defined at top level of spec".format(self._input_file))
             e = InputFileNotFoundError(self._input_file, search_dirs)
             self._input_file = find(self._input_file, search_dirs)
             if not self._input_file:
                 raise e
-
         for plot in self.subplots:
             plot.find_input_files(search_dirs)
 
